@@ -1,18 +1,17 @@
 from unittest import IsolatedAsyncioTestCase, TestCase
 
 from aiohttp.test_utils import make_mocked_request
-from heaobject.aws import S3StorageClass, S3ArchiveDetailState
+from heaobject.aws import AWSDesktopObject, S3StorageClass, S3ArchiveDetailState
 from heaobject.data import AWSS3FileObject
-from heaobject.group import SUPERADMIN_GROUP
 from heaobject.root import Permission
-from moto import mock_aws
 from heaserver.service.db import awsservicelib
 from botocore.exceptions import ClientError
 
 import heaserver.service.db.aws
 from heaserver.service.db.aws import S3ObjectPermissionContext
 from heaserver.service.testcase.mockaws import MockS3
-
+from collections.abc import Sequence
+from types import MethodType
 
 class TestAWSServiceLib(TestCase):
 
@@ -30,19 +29,22 @@ class TestAWSServiceLib(TestCase):
         self.assertEqual(500, awsservicelib.handle_client_error(c).status)
 
 
-class TestAsyncAWSServiceLib(IsolatedAsyncioTestCase):
-    @mock_aws()
-    def run(self, result):
-        super().run(result)
+class TestArchiveStatus(IsolatedAsyncioTestCase):
 
     def setUp(self):
         self.request = make_mocked_request('GET', '/')
         self.request.app['HEA_db'] = MockS3(None)
         self.perm_context: S3ObjectPermissionContext = S3ObjectPermissionContext(self.request)
-        from types import MethodType
+
         async def mock_is_super_admin(self) -> bool:
             return False
+        async def mock_simulate_perms(self, obj: AWSDesktopObject, actions: Sequence[str]) -> list[Permission]:
+            return [Permission.VIEWER, Permission.EDITOR, Permission.DELETER]
+        async def mock_is_account_owner(self) -> bool:
+            return False
         self.perm_context.is_super_admin = MethodType(mock_is_super_admin, self.perm_context)
+        self.perm_context._simulate_perms = MethodType(mock_simulate_perms, self.perm_context)
+        self.perm_context.is_account_owner = MethodType(mock_is_account_owner, self.perm_context)
 
     async def test_handle_display_name_attribute_permissions_glacier_deep_archive(self):
         obj: AWSS3FileObject = AWSS3FileObject()
