@@ -1,4 +1,5 @@
 from abc import ABC, abstractmethod
+from dataclasses import dataclass
 from functools import partial
 from aiohttp.web import Request
 from aiohttp import ClientResponseError
@@ -7,6 +8,7 @@ import botocore
 from aiohttp import hdrs, web
 from botocore.exceptions import ClientError, ParamValidationError
 from mypy_boto3_iam import IAMClient
+from mypy_boto3_iam.type_defs import PolicyDocumentDictTypeDef
 from mypy_boto3_s3 import S3Client
 from mypy_boto3_sqs import SQSClient
 from mypy_boto3_sts import STSClient
@@ -19,7 +21,7 @@ from ..oidcclaimhdrs import SUB
 from ..sources import AWS as AWS_SOURCE
 from heaobject.root import Permission, Share, ShareImpl, DesktopObject
 from heaobject.aws import S3Object, AWSDesktopObject, AmazonResourceName
-from heaobject.account import AWSAccount
+from heaobject.account import AWSAccount, AWSAccountCollaborators
 from heaobject.keychain import AWSCredentials, Credentials
 from heaobject.registry import Property
 from heaobject.volume import AWSFileSystem, FileSystem
@@ -30,17 +32,18 @@ from ..util import async_retry, now, LockManager
 from ..aiohttp import extract_sub
 from .. import client
 from yarl import URL
-from typing import Optional, TypeVar, cast, overload, Literal, Generic, TypedDict, NotRequired, Unpack, Callable
+from typing import Optional, TypeVar, cast, overload, Literal, TypedDict, NotRequired, Unpack
 from configparser import ConfigParser
 import asyncio
 from threading import Lock
-from collections.abc import Sequence, AsyncIterator
+from collections.abc import AsyncGenerator, Iterable, Sequence, AsyncIterator
 from copy import copy, deepcopy
 from .awsaction import *
 from cachetools import TTLCache
 from datetime import timedelta
 from base64 import urlsafe_b64encode
 from cachetools import TTLCache
+from collections import defaultdict
 import logging
 
 CLIENT_ERROR_NO_SUCH_BUCKET = 'NoSuchBucket'
@@ -71,7 +74,9 @@ _permission_for = {
     S3_CREATE_BUCKET: Permission.EDITOR,
     S3_DELETE_BUCKET: Permission.DELETER,
     S3_GET_BUCKET_TAGGING: Permission.VIEWER,
-    S3_PUT_BUCKET_TAGGING: Permission.EDITOR
+    S3_PUT_BUCKET_TAGGING: Permission.EDITOR,
+    S3_LIST_OBJECT_VERSIONS: Permission.VIEWER,
+    S3_DELETE_OBJECT_VERSION: Permission.DELETER
 }
 
 AWSDesktopObjectTypeVar = TypeVar('AWSDesktopObjectTypeVar', bound=AWSDesktopObject)
@@ -200,11 +205,12 @@ class AWSPermissionContext(HEAServerPermissionContext, ABC):
             loop = asyncio.get_running_loop()
             try:
                 assert credentials.role is not None, 'role attribute cannot be None'
-                logger.debug('Checking permissions')
+                caller_arn = self._caller_arn(obj)
+                logger.debug('Checking permissions for caller ARN %s for policy source ARN %s and actions %s', caller_arn, credentials.role, actions)
                 response_ = await loop.run_in_executor(None, partial(iam_client.simulate_principal_policy,
                                                                      PolicySourceArn=credentials.role,
                                                                      ActionNames=actions,
-                                                                     CallerArn=self._caller_arn(obj)))
+                                                                     ResourceArns=[caller_arn] if not caller_arn.endswith('/') else [caller_arn + '*']))
                 logger.debug('Response for checking %s: %s', actions, response_)
                 for results in response_['EvaluationResults']:
                     if results['EvalDecision'] == 'allowed':
@@ -274,7 +280,8 @@ class S3ObjectPermissionContext(AWSPermissionContext):
             return []
         if attr == 'tags' and not await self.is_account_owner():
             return await self._simulate_perms(obj, [S3_GET_OBJECT_TAGGING, S3_PUT_OBJECT_TAGGING])
-        elif attr == 'display_name' and (retrievable := getattr(obj, 'retrievable', None)) is not None and not retrievable:
+        elif attr == 'display_name' and (((retrievable := getattr(obj, 'retrievable', None)) is not None and not retrievable) or \
+                (Permission.DELETER not in await self._simulate_perms(obj, [S3_DELETE_OBJECT_VERSION]))):
             # Archived objects and objects in the process of being restored can only be viewed. For objects where the
             # archive status is unknown, we fall back to the default permissions, which is appropriate for folders, for
             # which archive status does not apply. This could result in a file's display_name getting write permissions
@@ -478,10 +485,18 @@ class S3(Database):
 
         if account is None:
             try:
-                async with STSClientContext(request, volume_id=volume_id, credentials=credentials) as sts_client:
+                async with STSClientContext(request, credentials=credentials) as sts_client:
                     account = await self._get_basic_account_info(sts_client)
-                    self.__account_cache[key] = account
                     aid = account.id
+                async with S3ClientContext(request, credentials=credentials) as s3_client:
+                    buckets = await asyncio.to_thread(s3_client.list_buckets)
+                elevated_credentials = await request.app[HEA_DB].elevate_privileges(request, credentials) if credentials is not None else None
+                async with IAMClientContext(request, credentials=elevated_credentials) as iam_client:
+                    bucket_names = (bucket_info['Name'] for bucket_info in buckets.get('Buckets', []))
+                    async for bucket_name, collaborator_ids in _get_bucket_collaborators(iam_client, bucket_names):
+                        account.add_collaborators_to(AWSAccountCollaborators(bucket_id=bucket_name,
+                                                                              collaborator_ids=collaborator_ids))
+                self.__account_cache[key] = account
             except ClientError:
                 logger.debug(f'Error getting account for volume {volume_id}', exc_info=True)
                 return None
@@ -1140,7 +1155,7 @@ def client_error_status(e: ClientError) -> tuple[int, str]:
     if error_code in (CLIENT_ERROR_404, CLIENT_ERROR_NO_SUCH_BUCKET, CLIENT_ERROR_NO_SUCH_KEY):  # folder doesn't exist
         return 404, ''
     elif error_code in (CLIENT_ERROR_ACCESS_DENIED, CLIENT_ERROR_ACCESS_DENIED2, CLIENT_ERROR_FORBIDDEN, CLIENT_ERROR_ALL_ACCESS_DISABLED):
-        return 403, ''
+        return 403, 'Access denied'
     elif error_code in (CLIENT_ERROR_INVALID_OBJECT_STATE, CLIENT_ERROR_BUCKET_NOT_EMPTY):
         return 400, str(e)
     else:
@@ -1182,3 +1197,73 @@ def add_default_share_to_desktop_object(sub: str, obj: AWSDesktopObject):
     """
     if sub != NONE_USER:
         obj.add_share(get_default_share(sub))
+
+
+@dataclass
+class _GetCollaboratorRetval:
+    bucket_name: str
+    collaborator_id: str
+    other_bucket_names: list[str]
+
+
+async def _get_collaborators(iam_client: IAMClient, buckets: Iterable[str]) -> AsyncGenerator[tuple[str, _GetCollaboratorRetval], None]:
+    logger = logging.getLogger(__name__)
+    logger.debug('Getting collaborators')
+    bucket_set = set(buckets)
+    try:
+        async for policy_info in _all_collaborator_policies_gen(iam_client):
+            for bucket_name in policy_info.bucket_names:
+                if bucket_name in bucket_set:
+                    yield bucket_name, _GetCollaboratorRetval(bucket_name=bucket_name,
+                                                            collaborator_id=policy_info.user_id,
+                                                            other_bucket_names=list(policy_info.bucket_names - set([bucket_name])))
+    except ClientError as e:
+        if e.response['Error']['Code'] != 'NoSuchBucketPolicy':
+            logging.exception('Error getting collaborators')
+            raise e
+
+
+async def _get_bucket_collaborators(iam_client: IAMClient, buckets: Iterable[str]) -> AsyncGenerator[tuple[str, list[str]], None]:
+    bucket_to_collab = defaultdict[str, list[str]](list)
+    async for _, collab_info in _get_collaborators(iam_client, buckets):
+        bucket_to_collab[collab_info.bucket_name].append(collab_info.collaborator_id)
+    for bucket_name, collab_ids in bucket_to_collab.items():
+        yield bucket_name, collab_ids
+
+import re
+_collaborator_policy_name_prefix = 'aws-Collaborator.Policy_'
+_arn_pattern = re.compile("[:/]")
+
+@dataclass
+class _PolicyInfo:
+    user_id: str
+    policy_arn: str
+    bucket_names: set[str]
+    policy_doc: PolicyDocumentDictTypeDef
+
+
+async def _all_collaborator_policies_gen(iam_client: IAMClient) -> AsyncGenerator[_PolicyInfo, None]:
+    logger = logging.getLogger(__name__)
+    logger.debug('Getting collaborator policy')
+    loop = asyncio.get_running_loop()
+    response_ = (await loop.run_in_executor(None, partial(iam_client.get_account_authorization_details,
+                                                                     Filter=['LocalManagedPolicy'])))
+    logger.debug('Account authorization details: %s', response_)
+    for policy in response_['Policies']:
+        logger.debug('Policy detail: %s', policy)
+        if policy['PolicyName'].startswith(_collaborator_policy_name_prefix):
+            user_id = policy['PolicyName'].removeprefix(_collaborator_policy_name_prefix)
+            logger.debug('User id %s', user_id)
+            for version in policy['PolicyVersionList']:
+                if version['IsDefaultVersion']:
+                    pol_doc = version['Document']
+                    assert not isinstance(pol_doc, str), 'Unexpected policy document type'
+                    pol_doc_stmt = pol_doc['Statement']
+                    assert not isinstance(pol_doc_stmt, str), 'Unexpected policy document statement type'
+                    assert len(pol_doc_stmt) == 2, 'Unexpected number of policy statements'
+                    bucket_names: set[str] = set()
+                    for resource in pol_doc_stmt[0]['Resource']:
+                        bucket_names.add(_arn_pattern.split(resource)[5])
+                    logger.debug('Returning policy doc from policy %s for user %s with buckets %s', policy, pol_doc, bucket_names)
+                    assert not isinstance(pol_doc, str), 'pol_doc unexpected type'
+                    yield _PolicyInfo(user_id=user_id, policy_arn=policy['Arn'], bucket_names=bucket_names, policy_doc=pol_doc)
