@@ -7,9 +7,10 @@ S3_LIST_BUCKET = 's3:ListBucket'
 S3_CREATE_BUCKET = 's3:CreateBucket'
 S3_DELETE_BUCKET = 's3:DeleteBucket'
 S3_GET_BUCKET_LOCATION = 's3:GetBucketLocation'  # Permission to get the bucket's region
-S3_GET_BUCKET_OBJECT_LOCK_CONFIGURATION = 'S3:GetBucketObjectLockConfiguration'
-S3_GET_BUCKET_VERSIONING = 'S3:GetBucketVersioning'
+S3_GET_BUCKET_OBJECT_LOCK_CONFIGURATION = 's3:GetBucketObjectLockConfiguration'
 S3_GET_BUCKET_TAGGING = 's3:GetBucketTagging'
 S3_PUT_BUCKET_TAGGING = 's3:PutBucketTagging'
-S3_GET_BUCKET_VERSIONING = 'S3:GetBucketVersioning'
+S3_GET_BUCKET_VERSIONING = 's3:GetBucketVersioning'
 S3_PUT_BUCKET_VERSIONING = 's3:PutBucketVersioning'
+S3_LIST_OBJECT_VERSIONS = 's3:ListBucketVersions'
+S3_DELETE_OBJECT_VERSION = 's3:DeleteObjectVersion'
