@@ -1,15 +1,13 @@
 import abc
 import configparser
 import asyncio
-import inspect
 from abc import ABC, abstractmethod
-from contextlib import AbstractAsyncContextManager, ExitStack, contextmanager, AbstractContextManager
+from contextlib import AbstractAsyncContextManager, ExitStack, asynccontextmanager, contextmanager, AbstractContextManager
 from types import TracebackType
 
 from aiohttp import web
-from typing import Any, Mapping, Optional, Type, TypeVar, Union, Protocol, Generic, Awaitable, runtime_checkable, \
-    overload
-from collections.abc import Mapping, Generator, AsyncIterator, Sequence
+from typing import Any, Mapping, Optional, Type, TypeVar, Union, Protocol, Generic, Awaitable, runtime_checkable
+from collections.abc import AsyncGenerator, Mapping, AsyncIterator, Sequence
 from aiohttp.web import Request
 
 from aiohttp.web_request import Request
@@ -61,13 +59,15 @@ class Database(abc.ABC):
     inheritance.
     """
 
-    def __init__(self, config: Optional[configparser.ConfigParser], **kwargs) -> None:
+    def __init__(self, config: Optional[configparser.ConfigParser], managed = False, **kwargs) -> None:
         """
         Initializes the database object. Subclasses must call this constructor.
 
         :param config: Optional HEA configuration.
+        :param managed: Whether this database instance is managed by a database manager.
         """
         super().__init__(**kwargs)
+        self.__managed = managed
         logger = logging.getLogger(__name__)
         if logger.getEffectiveLevel() == logging.DEBUG:
             logger.debug('Initialized %s database object', type(self))
@@ -182,12 +182,24 @@ class Database(abc.ABC):
         """
         return PermissionContext(request.headers.get(SUB, NONE_USER))
 
-    def close(self):
+    def really_close(self):
         """
-        Closes the database client. This method may be called multiple times.
+        Cleans up all resources associated with the database connection. This method may be called multiple times. The
+        default implementation does nothing.
         """
         pass
 
+    def close(self):
+        """Cleans up resources associated with database connections. This method may omit closing some clients, for
+        example when it's more efficient to keep the client open for the lifetime of an application. When closing the
+        application, call the really_close() method to close everything. The default implementation does nothing."""
+        pass
+
+    @property
+    def managed(self) -> bool:
+        """Whether this database instance is managed by a database manager."""
+        return self.__managed
+
 
 class NoDatabase(Database):
     def __init__(self):
@@ -545,9 +557,8 @@ class DatabaseManager(ABC):
 
     def close(self):
         """
-        Removes all data and content, and cleans up any other resources created
-        by other methods of this object. This method is not designed to be
-        overridden. This method may be called multiple times.
+        Removes all data and content, and cleans up any other resources created by other methods of this object. This
+        method is not designed to be overridden. This method may be called multiple times.
         """
         if self.started:
             try:
@@ -557,7 +568,7 @@ class DatabaseManager(ABC):
                     exceptions = []
                     while self.__databases:
                         try:
-                            self.__databases.pop().close()
+                            self.__databases.pop().really_close()
                         except Exception as e:
                             exceptions.append(e)
                 finally:
@@ -620,8 +631,8 @@ class DatabaseManager(ABC):
         """
         pass
 
-    @contextmanager
-    def database(self) -> Generator[Database, None, None]:
+    @asynccontextmanager
+    async def database(self) -> AsyncGenerator[Database, None]:
         """
         Context manager that creates the corresponding Database object.
         Override this method to return a Database instance. The database object
@@ -635,7 +646,7 @@ class DatabaseManager(ABC):
         try:
             yield db
         finally:
-            db.close()
+            await asyncio.to_thread(db.really_close)
 
 class MicroserviceDatabaseManager(DatabaseManager, ABC):
     """
@@ -752,11 +763,7 @@ class DatabaseContextManager(Generic[DatabaseConnectionTypeVar, CredentialTypeVa
         :param exc_value: the exception itself, if any.
         :param traceback: the traceback, if any.
         """
-        close_method = self.__connection.close
-        result = close_method()
-
-        if inspect.isawaitable(close_method):
-            await result
+        self.__connection.close()
 
     @abstractmethod
     async def connection(self) -> DatabaseConnectionTypeVar:
