@@ -88,12 +88,15 @@ class Evaluator:
             raise EvaluatorException(f'Assignment {expr} attempted')
 
 
-def get_eval_for(obj: HEAObject | HEAObjectDict, extra_names: Mapping[str, Any] | None = None) -> EvalProtocol:
+def get_eval_for(obj: HEAObject | HEAObjectDict, extra_functions: Mapping[str, Any] | None = None, extra_names: Mapping[str, Any] | None = None) -> EvalProtocol:
     """
     Creates an expression evaluator, customized for HEA. Loads the provided HEAObject, as well as any provided extra
     names.
 
     :param obj: the HEAObject to load into the evaluator (required).
+    :param extra_functions: a mapping of function names to functions to make available to the evaluator. It automatically
+    adds the isheaobject function, which returns whether the object is an instance of the provided class. Passing a
+    function with the same name will override the default isheaobject function.
     :param extra_names: any extra names to make available to the evaluator.
     :return: an object implementing the EvalProtocol protocol.
 
@@ -121,5 +124,8 @@ def get_eval_for(obj: HEAObject | HEAObjectDict, extra_names: Mapping[str, Any]
         else:
             raise ValueError(f'cls {cls} is not a HEAObject')
 
-    return Evaluator(SimpleEval(functions={'isheaobject': isheaobject}, names=names))
+    functions = {'isheaobject': isheaobject}
+    functions.update(extra_functions or {})
+
+    return Evaluator(SimpleEval(functions=functions, names=names))
 
