@@ -75,9 +75,12 @@ async def subscribe(hostname=DEFAULT_HOSTNAME,
                     async with message.process():
                         message_body = message.body
                         logger.debug('Received message body: %s', message_body)
-                        hea_object = desktop_object_from_json(message_body)
-                        assert isinstance(hea_object, tuple(desktop_object_type_for_name(tn) for tn in topics)), f'Expected type {topics} but received {type(hea_object)}'
-                        yield hea_object
+                        try:
+                            hea_object = desktop_object_from_json(message_body)
+                            assert isinstance(hea_object, tuple(desktop_object_type_for_name(tn) for tn in topics)), f'Expected type {topics} but received {type(hea_object)}'
+                            yield hea_object
+                        except:
+                            logger.exception('Error processing message body')
         finally:
             logger.debug('Closing the channel and connection...')
             await channel.close()
