@@ -567,8 +567,8 @@ async def _handle_get_result(request: web.Request, data: Union[DesktopObjectDict
     response body. Ensure that the request parameters and data parameter contain any information needed to generate the
     links. If include_data is False, the permissions and attribute_permissions parameters are ignored and should be
     omitted.
-    :return: aiohttp.web.Response object, either a 200 status code and the requested JSON object in the body, or status
-    code 404 (Not Found).
+    :return: aiohttp.web.Response object, either a 200 status code and the requested JSON object in the body, status
+    code 404 (Not Found), or 500 (Internal Server Error) if the data is invalid.
     """
     logger = logging.getLogger(__name__)
     if data is not None:
