@@ -204,7 +204,7 @@ async def expected_one(fixtures: Mapping[CollectionKey | str, list[DesktopObject
     """
     desktop_object_dict = query_fixture_collection(fixtures, coll)[0]
     id_ = str(desktop_object_dict['id'])
-    href = URL(wstl_builder.href if wstl_builder.href else '') / id_
+    href_encoded = URL(wstl_builder.href if wstl_builder.href else '') / id_
     get_actions_ = get_actions or []
 
     def item_links() -> list[dict[str, Any]]:
@@ -216,15 +216,15 @@ async def expected_one(fixtures: Mapping[CollectionKey | str, list[DesktopObject
             if itemif is not None and not get_eval_for(desktop_object_dict).eval(itemif):
                 continue
             if url is not None:
-                match = extra_tvars(route=url, url=str(href))
+                match = extra_tvars(route=url, url=str(href_encoded))
             else:
                 match = {}
             targets = action.get('target', '').split()
             if 'item' in targets and 'read' in targets and 'cj' in targets:
                 links.append({
                     'prompt': action['prompt'],
-                    'href': uritemplate.expand(url, {k: v for k, v in (match | desktop_object_dict).items() if
-                                                     isinstance(v, (int, float, str))}) if url else str(href),
+                    'href': uritemplate.expand(url, {k: v for k, v in (desktop_object_dict | match).items() if
+                                                     isinstance(v, (int, float, str))}) if url else str(href_encoded),
                     'rel': ' '.join(rel)
                 })
         return links
@@ -235,15 +235,15 @@ async def expected_one(fixtures: Mapping[CollectionKey | str, list[DesktopObject
             if action is None:
                 raise ValueError(f'Invalid action name {action_name}')
             if url is not None:
-                match = extra_tvars(route=url, url=URL(href).path)
+                match = extra_tvars(route=url, url=URL(href_encoded).path)
             else:
                 match = {}
             targets = action.get('target', '').split()
             if 'item' in targets and 'href' in targets and 'cj' in targets:
                 return {
                     'prompt': action['prompt'],
-                    'href': uritemplate.expand(url, {k: v for k, v in (match | desktop_object_dict).items() if
-                                                     isinstance(v, (int, float, str))}) if url else str(href),
+                    'href': uritemplate.expand(url, {k: v for k, v in (desktop_object_dict | match).items() if
+                                                     isinstance(v, (int, float, str))}) if url else str(href_encoded),
                     'rel': ' '.join(rel),
                     'readOnly': 'true'
                 }
@@ -256,7 +256,7 @@ async def expected_one(fixtures: Mapping[CollectionKey | str, list[DesktopObject
             if action is None:
                 raise ValueError(f'Invalid action name {action_name} in get_actions')
             if url is not None:
-                match = extra_tvars(route=url, url=URL(href).path)
+                match = extra_tvars(route=url, url=URL(href_encoded).path)
             else:
                 match = {}
             targets = action.get('target', '').split()
@@ -265,7 +265,7 @@ async def expected_one(fixtures: Mapping[CollectionKey | str, list[DesktopObject
                 match['id'] = id_
                 links.append({
                     'prompt': action['prompt'],
-                    'href': uritemplate.expand(url, match) if url else str(href),
+                    'href': uritemplate.expand(url, match) if url else str(href_encoded),
                     'rel': ' '.join(rel)
                 })
         return links
@@ -313,7 +313,7 @@ async def expected_one(fixtures: Mapping[CollectionKey | str, list[DesktopObject
         _data_append(data_, x, y)
     collection: dict[str, Any] = {
         'collection': {
-            'href': str(href),
+            'href': str(href_encoded),
             'permissions': [[perm.name for perm in perms]],
             'items': [{'data': data_,
                        'links': item_links()}],
@@ -493,7 +493,7 @@ async def expected_all(fixtures: Mapping[CollectionKey | str, list[DesktopObject
             if 'item' in targets and 'read' in targets and 'cj' in targets:
                 links.append({
                     'prompt': action['prompt'],
-                    'href': uritemplate.expand(route, match | obj) if route else str(URL(href_) / obj['id']),
+                    'href': uritemplate.expand(route, obj | match) if route else str(URL(href_) / obj['id']),
                     'rel': ' '.join(rel)
                 })
         return links
@@ -511,7 +511,7 @@ async def expected_all(fixtures: Mapping[CollectionKey | str, list[DesktopObject
             if 'item' in targets and 'href' in targets and 'cj' in targets:
                 return {
                     'prompt': action['prompt'],
-                    'href': uritemplate.expand(url, match | obj) if url else str(URL(href_) / obj['id']),
+                    'href': uritemplate.expand(url, obj | match) if url else str(URL(href_) / obj['id']),
                     'rel': ' '.join(rel),
                     'readOnly': 'true'
                 }
@@ -640,14 +640,17 @@ def _create_template(d: DesktopObjectDict, exclude=('id',)) -> dict[str, dict[st
                                   _nvpjson_property_to_cj_part_iterator(x, y)]}}
 
 
-def _template_data_iterator(sub: str, action: Mapping[str, Any], desktop_object: DesktopObject,
+def _template_data_iterator(sub: str, action: wstl.WeSTLactionType, desktop_object: DesktopObject,
                             attr_perms: Mapping[str, Sequence[Permission] | None], len_fixtures=1) -> Iterator[dict[str, Any]]:
     targets = action['target'].split()
+    desktop_object_dict = desktop_object.to_dict()
+    match = extra_tvars(route=str(action.get('url', '')), url='http://localhost:8080')
     for input in action.get('inputs', []):
         section = wstl.get_section(input)
         if optionsFromUrl := wstl.get_extended_property_value('optionsFromUrl', input):
             if 'href' not in optionsFromUrl:
-                optionsFromUrlPath = optionsFromUrl['path']
+                optionsFromUrlPath = uritemplate.expand(optionsFromUrl['path'], {k: v for k, v in (desktop_object_dict | match).items() if
+                                                        isinstance(v, (int, float, str))}) if len_fixtures == 1 else optionsFromUrl['path']
                 optionsFromUrl['href'] = 'http://localhost:8080' + ('/' if optionsFromUrlPath else '') + optionsFromUrlPath
         perms = attr_perms.get(input['name']) or []
         if 'add' in targets:
@@ -723,7 +726,7 @@ def _new_template_data_item(i: dict[str, Any], value: Any, sub: str, perms: Sequ
     return rtn
 
 
-def _set_collection_template(action: Mapping[str, Any], collection_doc: Mapping[str, Any],
+def _set_collection_template(action: wstl.WeSTLactionType, collection_doc: Mapping[str, Any],
                              desktop_object: DesktopObject, len_desktop_objects: int, rel: Iterable[str] | None,
                              sub: str, attr_perms: Mapping[str, Sequence[Permission] | None]):
     """
@@ -782,7 +785,7 @@ async def _expected_one_form(
             match = extra_tvars(route=a.url, url=str(href))
         else:
             match = {}
-        return {'href': uritemplate.expand(a.url, {k: v for k, v in (match | obj_dict).items()
+        return {'href': uritemplate.expand(a.url, {k: v for k, v in (obj_dict | match).items()
                                                    if isinstance(v, (int, float, str))}) if a.url else str(href),
                 'rel': ' '.join(a.rel or []),
                 'prompt': action_['prompt']}
