@@ -305,7 +305,7 @@ class MockS3Manager(S3Manager):
         return result
 
     def get_database(self) -> S3:
-        return MockS3(self.config)
+        return MockS3(self.config, managed=True)
 
     def insert_desktop_objects(self, desktop_objects: Optional[Mapping[CollectionKey, Sequence[DesktopObjectDict]]]):
         super().insert_desktop_objects(desktop_objects)
@@ -430,7 +430,7 @@ class MockS3WithMockMongoManager(MockS3Manager, MockMongoManager):
     """
 
     def get_database(self) -> MockS3WithMockMongo:
-        return MockS3WithMockMongo(config=self.config, mongo=self.get_mongo())
+        return MockS3WithMockMongo(config=self.config, mongo=self.get_mongo(), managed=True)
 
     @classmethod
     def database_types(self) -> list[str]:
