@@ -1,20 +1,22 @@
 import logging
+from typing import Any
 
 import regex as re
 from contextlib import suppress
 from collections.abc import Iterator
-from uritemplate.variable import VariableValueDict
+from urllib.parse import unquote
 
 TEMPLATE_NAMES_PATTERN = re.compile("\\{([\\w\\?;][-\\w\\.,]*)\\}")
 
 
-def tvars(route: str, url: str) -> VariableValueDict:
+def tvars(route: str, url: str) -> dict[str, Any]:
     """
     Get the variables from a URL given the route template. If the URL does not match the given route, then return {}.
 
     :param route: the route template.
     :param url: the url that may be a valid one for the given route.
     :return: a dictionary containing references to all the variables from the route that were given in curly braces.
+    The variable values are decoded if necessary.
     :raises ValueError: if an error occurs trying to compute the variables.
     """
     try:
@@ -24,7 +26,7 @@ def tvars(route: str, url: str) -> VariableValueDict:
             route = route[:-1]
             with suppress(Exception):
                 match = _to_regex(route).fullmatch(url, partial=True)
-        return match.groupdict() if match else {}
+        return {k: unquote(v) for k, v in match.groupdict().items()} if match else {}
     except re.error as e:
         raise ValueError from e
 
